VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 1  'NoTransaction
END
Attribute VB_Name = "Bank"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Class module in business tier

Const MODULE_NAME As String = "Bank_Bus.Bank"

Public Property Get MaxBadTries() As Integer
' Count for maximum login failures before locking out the account
  MaxBadTries = 5
End Property

Public Function VerifyLogin(ByVal BankID As Integer, _
  ByVal CardID As Long, ByVal Password As String, _
  ByRef InvalidCount As Variant) As Boolean
' Verify a valid login

  Dim objBank As Bank_DB.Bank

  On Error GoTo ErrorHandler
  
  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  VerifyLogin = objBank.VerifyLogin(BankID, CardID, _
    Password, InvalidCount)
  
  CtxSetComplete 'Commit the transaction

Clean_Up:
  Set objBank = Nothing
  Exit Function

ErrorHandler:
  RaiseError MODULE_NAME, "VerifyLogin(" & CStr(BankID) & _
    ", " & CStr(CardID) & ", " & Password & ")"
  GoTo Clean_Up
End Function

Public Property Get ChangeLogin(ByVal BankID As Integer, _
  ByVal CardID As Long, ByVal Password As String) As Boolean
' Verify a valid login

  Dim objBank As Bank_DB.Bank

  On Error GoTo ErrorHandler

  ' Create a transaction context for the DB call
  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  ChangeLogin = objBank.ChangeLogin(BankID, CardID, Password)
  
  CtxSetComplete 'Commit the transaction
  
Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "ChangeLogin(" & CStr(BankID) & _
    ", " & CStr(CardID) & ", " & Password & ")"
  GoTo Clean_Up
End Property

Public Sub UpdatePasswordCount(ByVal BankID As Integer, _
  ByVal CardID As Long, ByVal InvalidCount As Integer)
' Update the count of failed login attempts

  Dim objBank As Bank_DB.Bank

  On Error GoTo ErrorHandler
   
  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  objBank.UpdatePasswordCount BankID, CardID, InvalidCount
  
  CtxSetComplete
  
Clean_Up:
  Set objBank = Nothing
  Exit Sub

ErrorHandler:
  RaiseError MODULE_NAME, "UpdatePasswordCount(" & CStr(BankID) _
    & ", " & CStr(CardID) & ", " & CStr(InvalidCount) & ")"
  GoTo Clean_Up
End Sub

Public Property Get WROBALock(ByVal BankID As Integer, _
  ByVal CardID As Long) As Boolean
' Lock the account

  Dim objBank As Bank_DB.Bank
  
  On Error GoTo ErrorHandler

  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  ' Pessimistically assume failure...
  WROBALock = False
  
  'Passthrough call to data tier
  objBank.WROBALock BankID, CardID
  
  CtxSetComplete
  WROBALock = True

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  Select Case Err.Number
  Case -2147217900
    Err.Clear
  Case Else
    RaiseError MODULE_NAME, "WROBALock(" & CStr(BankID) & _
      ", " & CStr(CardID) & ")"
  End Select
  GoTo Clean_Up
End Property

Public Sub WROBAUnLock(ByVal BankID As Integer, _
  ByVal CardID As Long)
' Unlock the account

  Dim objBank As Bank_DB.Bank

  On Error GoTo ErrorHandler

  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  objBank.WROBAUnLock BankID, CardID
  
  CtxSetComplete
  
Clean_Up:
  Set objBank = Nothing
  Exit Sub

ErrorHandler:
  RaiseError MODULE_NAME, "WROBAUnLock(" & CStr(BankID) & _
    ", " & CStr(CardID) & ")"
  GoTo Clean_Up
End Sub

Public Property Get GetAccounts(ByVal BankID As Integer, _
  ByVal CardID As Long) As ADODB.Recordset
' Get the accounts linked to this CardID

  Dim objBank As Bank_DB.Bank
  
  On Error GoTo ErrorHandler

  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  Set GetAccounts = objBank.GetAccounts(BankID, CardID)
  
  CtxSetComplete
  
Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "GetAccounts(" & CStr(BankID) & _
    ", " & CStr(CardID) & ")"
  GoTo Clean_Up
End Property

Public Sub GetAccountBalance(ByVal BankID As Integer, _
  ByVal AccountID As Long, ByRef AccountTypeDesc As Variant, _
  ByRef Balance As Variant, ByRef DailyRemaining As Variant, _
  ByRef OverdraftLimit As Variant)
' Get the balance for the AccountID account

  Dim objBank As Bank_DB.Bank

  On Error GoTo ErrorHandler
  
  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  objBank.GetAccountBalance BankID, AccountID, AccountTypeDesc, _
    Balance, DailyRemaining, OverdraftLimit
  
  CtxSetComplete
  
Clean_Up:
  Set objBank = Nothing
  Exit Sub

ErrorHandler:
  RaiseError MODULE_NAME, "GetAccountBalance(" & CStr(BankID) _
    & ", " & CStr(AccountID) & ", ... " & ")"
  GoTo Clean_Up
End Sub

Public Property Get GetAccountHistory(ByVal BankID As Integer, _
  ByVal AccountID As Long, ByVal MaxTransactionDate As Date) _
  As ADODB.Recordset
' Get the transaction history for this AccountID

  Dim objBank As Bank_DB.Bank
  
  On Error GoTo ErrorHandler

  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  Set GetAccountHistory = objBank.GetAccountHistory(BankID, _
    AccountID, MaxTransactionDate)
  
  CtxSetComplete

Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "GetAccountHistory(" & CStr(BankID) _
    & ", " & CStr(AccountID) & ", " & CStr(MaxTransactionDate) & ")"
  GoTo Clean_Up
End Property

Public Property Get GetBanks(ByVal BankID As Integer) As ADODB.Recordset
' Returns a recordset with all the banks in the Bank table
  Dim objBank As Bank_DB.Bank
  
  On Error GoTo ErrorHandler

  Set objBank = CtxCreateObject("Bank_DB.Bank")
  
  'Passthrough call to data tier
  Set GetBanks = objBank.GetBanks(BankID)
  
  CtxSetComplete
  
Clean_Up:
  Set objBank = Nothing
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "GetBanks(" & CStr(BankID) & ")"
  GoTo Clean_Up
End Property

Public Property Get GetCardID(strCardID As String) As Long
  ' The 9 rightmost digits are the WROBA Card ID. The DB tables
  ' are set up for the case study to use only the rightmost
  ' 5 digits. To enable the use of 9 digits the tables must
  ' be changed to a different data type.
  GetCardID = CLng(Right$(strCardID, 9))
End Property

Public Property Get GetBankID(strCardID As String) As Integer
  ' The 3 leftmost digits are the Bank ID. The DB tables
  ' are set up for the case study to use only the rightmost
  ' 1 digit of those 3 digits. To enable the use of 3 digits
  ' the tables must be changed to a different data type.
    GetBankID = CInt(Left$(strCardID, 3))
End Property
